function values(...values) {
    return (o, idx) => {
        if (idx < values.length) {
            return values[idx];
        }
        throw `idx=${idx} out of values=${values}`;
    };
}
function field(name) {
    return o => o[name];
}
function mapField(name, map) {
    return o => map.get(o[name]);
}
function joinFields(separator = '_', ...names) {
    return combineFields(joiner(separator), names);
}
function combineFields(combiner, names) {
    return o => combiner(names.map(f => o[f]).filter(nonUndefined));
}
function joinProviders(separator = '_', ...providers) {
    return combineProviders(joiner(separator), ...providers);
}
function combineProviders(combiner, ...providers) {
    return (o, i) => combiner(providers.map(p => p(o, i)).filter(nonUndefined));
}
function joiner(separator = '_') {
    return args => args.join(separator);
}
function rangeFields(minName, maxName) {
    return (o, i) => {
        let min = field(minName)(o, i);
        let max = field(maxName)(o, i);
        return nonUndefined(min) && nonUndefined(max) ? [min, max] : undefined;
    };
}
function buildSingleSerieChart(objects, provider, defaultValue) {
    let copy = provider;
    if (objects?.length > 1 && (provider.series?.length > 1 || typeof provider.series[0].name == 'function')) {
        copy = { ...provider,
            pivot: false,
            series: provider.series.map(s => ({
                data: {
                    x: (provider.pivot // TODO change that cast
                        ? combineProviders(joiner(), resolveDataProvider(s.name), s.data.x)
                        : combineProviders(joiner(), s.data.x, resolveDataProvider(s.name))),
                    y: s.data.y
                },
                color: s.color,
                //no unit
            }))
        };
    }
    return { ...buildChart(objects, copy, defaultValue), pivot: provider.pivot };
}
function buildChart(objects, provider, defaultValue) {
    var mappers = provider.pivot ?
        provider.series.map(m => ({
            name: resolveDataProvider(m.data.x),
            data: { x: resolveDataProvider(m.name, ''), y: m.data.y },
        })) : provider.series;
    var chart = newChart(provider);
    if (!provider.continue) {
        chart.categories = distinct(objects, mappers.map(m => m.data.x));
        if (provider.xorder) {
            chart.categories.sort(naturalComparator(provider.xorder));
        }
    }
    var series = {};
    mappers.forEach(m => {
        var np = resolveDataProvider(m.name);
        var sp = resolveDataProvider(m.stack);
        var cp = resolveDataProvider(m.color);
        var tp = resolveDataProvider(m.type);
        objects.forEach((o, i) => {
            var name = np(o, i) || ''; //can't use undefined as a map key
            if (!series[name]) { //init serie
                series[name] = { data: provider.continue ? [] : new Array(chart.categories.length).fill(defaultValue) };
                var stack = sp(o, i);
                var color = cp(o, i);
                var type = tp(0, i);
                name && (series[name].name = name);
                stack && (series[name].stack = stack);
                color && (series[name].color = color);
                type && (series[name].type = type);
            }
            if (provider.continue) {
                series[name].data.push({ x: m.data.x(o, i), y: requireNonUndefined(m.data.y(o, i), defaultValue) });
            }
            else {
                var key = m.data.x(o, i);
                var idx = chart.categories.indexOf(key);
                if (idx > -1) { //if !exist
                    series[name].data[idx] = requireNonUndefined(m.data.y(o, i), defaultValue);
                }
                else {
                    throw `'${key}' not part of categories : ${chart.categories}`;
                }
            }
        });
    });
    chart.series = Object.values(series);
    if (provider.continue && provider.xorder) {
        chart.series.forEach(s => s.data.sort(naturalFieldComparator(provider.xorder, field('x'))));
    }
    return chart;
}
function newChart(provider) {
    return Object.entries(provider)
        .filter(e => ['series'].indexOf(e[0]) < 0)
        .reduce((acc, e) => { acc[e[0]] = e[1]; return acc; }, { series: [] });
}
function resolveDataProvider(provider, defaultValue) {
    return !provider
        ? (o, i) => defaultValue
        : typeof provider == 'function'
            ? provider
            : (o, i) => provider;
}
function distinct(objects, providers) {
    var categs = new Set();
    providers.forEach(p => objects.forEach((o, i) => categs.add(p(o, i))));
    return [...categs];
}
function nonUndefined(o) {
    return !isUndefined(o);
}
function requireNonUndefined(o, elseValue) {
    return isUndefined(o) ? elseValue : o;
}
function isUndefined(o) {
    return o == undefined;
}
function naturalFieldComparator(sens, provider) {
    if (provider) {
        const v = sens == 'asc' ? 1 : -1;
        return (o1, o2) => {
            let a = provider(o1, undefined);
            let b = provider(o2, undefined);
            return a > b ? v : a < b ? -v : 0;
        };
    }
    return naturalComparator(sens);
}
function naturalComparator(sens) {
    const v = sens == 'asc' ? 1 : -1;
    return ((a, b) => a > b ? v : a < b ? -v : 0);
}
function groupByFiled(arr, name) {
    return groupBy(arr, field(name));
}
function groupBy(arr, fn) {
    return arr.reduce((acc, o, idx) => {
        var key = fn(o, idx);
        if (!acc[key]) {
            acc[key] = [];
        }
        acc[key].push(o);
        return acc;
    }, {});
}

/**
 * Simple object check.
 * @param item
 * @returns {boolean}
 */
function isObject(item) {
    return (item && typeof item === 'object' && !Array.isArray(item));
}
/**
* Deep merge two objects.
* @param target
* @param ...sources
*/
function mergeDeep(target, ...sources) {
    if (!sources.length)
        return target;
    const source = sources.shift();
    if (isObject(target) && isObject(source)) {
        for (const key in source) {
            if (isObject(source[key])) {
                if (!target[key])
                    Object.assign(target, { [key]: {} });
                mergeDeep(target[key], source[key]);
            }
            else {
                Object.assign(target, { [key]: source[key] });
            }
        }
    }
    return mergeDeep(target, ...sources);
}

/*
 * Public API Surface of jquery-core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { buildChart, buildSingleSerieChart, combineFields, combineProviders, distinct, field, groupBy, groupByFiled, isObject, joinFields, joinProviders, joiner, mapField, mergeDeep, naturalComparator, naturalFieldComparator, rangeFields, values };
//# sourceMappingURL=oneteme-jquery-core.mjs.map
