export declare type ChartType = 'line' | 'area' | 'pie' | "donut" | "radialBar" | "polarArea" | 'bar' | 'treemap' | 'funnel' | 'pyramid' | string;
export declare function values<T>(...values: T[]): DataProvider<T>;
export declare function field<T>(name: string): DataProvider<T>;
export declare function mapField<T>(name: string, map: Map<any, T>): DataProvider<T>;
export declare function joinFields(separator?: string, ...names: string[]): DataProvider<string>;
export declare function combineFields<T>(combiner: (args: any[]) => T, names: string[]): DataProvider<T>;
export declare function joinProviders<T>(separator?: string, ...providers: DataProvider<T>[]): DataProvider<string>;
export declare function combineProviders<T, R>(combiner: (args: T[]) => R, ...providers: DataProvider<T>[]): DataProvider<R>;
export declare function joiner(separator?: string): (args: any[]) => string;
export declare function rangeFields<T>(minName: string, maxName: string): DataProvider<T[]>;
export declare function buildSingleSerieChart<X extends XaxisType, Y extends YaxisType>(objects: any[], provider: ChartProvider<X, Y>, defaultValue?: Y): CommonChart<X, Y | Coordinate2D>;
export declare function buildChart<X extends XaxisType, Y extends YaxisType>(objects: any[], provider: ChartProvider<X, Y>, defaultValue?: Y): CommonChart<X, Y | Coordinate2D>;
export declare function distinct<T>(objects: any[], providers: DataProvider<T>[]): T[];
export interface ChartProvider<X extends XaxisType, Y extends YaxisType> {
    title?: string;
    subtitle?: string;
    xtitle?: string;
    ytitle?: string;
    width?: number;
    height?: number;
    stacked?: boolean;
    pivot?: boolean;
    continue?: boolean;
    xorder?: Sort;
    series?: SerieProvider<X, Y>[];
    options?: any;
}
export interface SerieProvider<X extends XaxisType, Y extends YaxisType> {
    data: CoordinateProvider<X, Y>;
    name?: string | DataProvider<string>;
    stack?: string | DataProvider<string>;
    color?: string | DataProvider<string>;
    type?: string | DataProvider<string>;
    unit?: string;
}
export declare type Coordinate2D = {
    x: XaxisType;
    y: YaxisType;
};
export declare type XaxisType = number | string | Date;
export declare type YaxisType = number | number[];
export declare type CoordinateProvider<X, Y> = {
    x: DataProvider<X>;
    y: DataProvider<Y>;
};
export declare type DataProvider<T> = (o: any, idx: number) => T;
export declare type Sort = 'asc' | 'desc';
export interface CommonChart<X extends XaxisType, Y extends YaxisType | Coordinate2D> {
    series: CommonSerie<Y>[];
    categories?: X[];
    title?: string;
    subtitle?: string;
    xtitle?: string;
    ytitle?: string;
    width?: number;
    height?: number;
    pivot?: boolean;
    continue?: boolean;
    stacked?: boolean;
    xorder?: Sort;
    options?: any;
}
export interface CommonSerie<Y extends YaxisType | Coordinate2D> {
    data: Y[];
    name?: string;
    stack?: string;
    color?: string;
}
export declare function naturalFieldComparator<T>(sens: Sort, provider: DataProvider<T>): (o1: any, o2: any) => number;
export declare function naturalComparator<T>(sens: Sort): (o1: T, o2: T) => number;
export declare function groupByFiled<T>(arr: [], name: string): {
    [key: string]: T[];
};
export declare function groupBy<T>(arr: [], fn: DataProvider<string>): {
    [key: string]: T[];
};
export interface ChartView<X extends XaxisType, Y extends YaxisType> {
    config: ChartProvider<X, Y>;
    data: any[];
    isLoading: boolean;
    canPivot?: boolean;
}
